#version 420 core 

layout(std140, binding = 1) uniform Material
{
    vec3 ka;
    vec3 kd;
    vec3 ks;
    float p;
    bool normalmapped;
    bool parallaxmapped;
};

layout (location = 0) out vec3 gPosition;
layout (location = 1) out vec3 gNormal;
layout (location = 2) out vec4 gDiffuseP;
layout (location = 3) out vec4 gAmbientSpecular;

in vec3 vertex; // Позиция вершины в пространстве
in vec3 N; // Нормаль трансформированноая
in vec2 texCoord; // Текстурные координаты
in vec3 T; // Касательный вектор
in vec3 B; // Бикасательный вектор
in vec3 view; // Вектор от поверхности к камере

uniform sampler2D tex_diffuse;
uniform sampler2D tex_ambient;
uniform sampler2D tex_specular;
uniform sampler2D tex_heights;
uniform sampler2D tex_normal;

uniform float parallax_heightScale = 0.03;

void main()
{    
    // Сформируем TBN матрицу
    mat3 TBN = mat3(T, B, N);
    // Перевод вектора в касательное пространство
    vec3 viewTBN = normalize(transpose(TBN) * view);
    // Измененные текстурные координаты
    vec2 new_texCoord = texCoord;

    // Сохранение позиции фрагмента в G-буфере
    gPosition = vertex;

    if (parallaxmapped)
    {
        float height = 1.0 - texture(tex_heights, texCoord).r;
        new_texCoord -= viewTBN.xy* (height * parallax_heightScale); 

        if(new_texCoord.x > 1.0 || new_texCoord.y > 1.0 || new_texCoord.x < 0.0 || new_texCoord.y < 0.0)
            discard;
    }

    // Сохранение нормали в G-буфере
    gNormal = N;
    // Если используется карта нормалей
    if (normalmapped)
    {
        // Получим значение из карты нормалей и приведем их к диапазону [-1;1]
        gNormal = texture(tex_normal, new_texCoord).rgb * 2 - 1.0f;  
        gNormal = normalize(TBN * gNormal); // Из касательного пространства в мировые координаты
    }

    // Сохранение диффузного цвета
    gDiffuseP.rgb = texture(tex_diffuse, new_texCoord).rgb * kd;
    // Сохранение глянцевости
    gDiffuseP.a = p;
    // Сохранение фоновой составляющей
    gAmbientSpecular.rgb = texture(tex_ambient, new_texCoord).rgb * ka;
    // Сохранение зеркальной составляющей
    gAmbientSpecular.a = texture(tex_specular, new_texCoord).r * ks.r;
}